/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlCudaDTree.cpp
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
**
*****************************************************************************/

#include "svlCudaDTree.h"

extern svlCudaMemHandler *cm;
typedef unsigned char uchar;

void printTree(int *iptr, float *fptr)
{
	/*printf("%d %0.4f %d %0.4f %0.4f %0.4f %d %0.4f %0.4f %0.4f %0.4f\n",
		*iptr, *fptr, *(iptr+1), *(fptr+1), *(fptr+1), *(fptr+1),
		*(iptr+1), *(fptr+1), *(fptr+1), *(fptr+1), *(fptr+1));*/
	printf("%d %d %d    %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f\n",
		*iptr, *(iptr+1), *(iptr+2),
		*fptr, *(fptr+2), *(fptr+3), *(fptr+4), *(fptr+5), *(fptr+6), *(fptr+7), *(fptr+8));
}

svlCudaDTree::svlCudaDTree()
{
}

// Read in model from file.
svlCudaDTree::svlCudaDTree(const char *file)
{
	FILE *fp = fopen(file,"rb");
	if ( !fp ) {
		printf("Initialized svlCudaDTree with NULL FILE: %s.\n", file);
		return;
	}

	// Read tree description in directly from file.
	fread(&n_trees,sizeof(int),1,fp);
	int *itree = (int*)malloc(sizeof(int)*n_trees*11);
	float *ftree = (float*)malloc(sizeof(float)*n_trees*11);
	int gi, *iptr = itree;
	float gf, *fptr = ftree;
	for ( int i=0; i<n_trees; ++i ) {
		fread(&gi,sizeof(int),1,fp); *iptr++ = gi;
		fread(&gf,sizeof(float),1,fp); *fptr++ = gf;
		fread(&gi,sizeof(int),1,fp); *iptr++ = gi;
		fread(&gf,sizeof(float),1,fp); *fptr++ = gf;
		fread(&gf,sizeof(float),1,fp); *fptr++ = gf;
		fread(&gf,sizeof(float),1,fp); *fptr++ = gf;
		fread(&gi,sizeof(int),1,fp); *iptr++ = gi;
		fread(&gf,sizeof(float),1,fp); *fptr++ = gf;
		fread(&gf,sizeof(float),1,fp); *fptr++ = gf;
		fread(&gf,sizeof(float),1,fp); *fptr++ = gf;
		fread(&gf,sizeof(float),1,fp); *fptr++ = gf;
	}
	fclose(fp);

	// Read in total tree.
	tree = (float*)malloc(sizeof(float)*n_trees*11);
	fp = fopen(file,"rb");
	fread(&gi,sizeof(int),1,fp);
	fread(tree,sizeof(float),n_trees*11,fp);
	fclose(fp);

	// Debug print
	/*for ( int i=0; i<n_trees; ++i )
		printTree(itree + i*3, ftree + i*8);*/

	// Allocate device memory and transfer in.
	dtree = cm->allocLine(n_trees*11);
	cm->lineToGPU(dtree, tree);

	//cm->line2file("line.out", dtree);
	
	free(itree);
	free(ftree);
}

svlCudaDTree::~svlCudaDTree()
{
	// Free host memory.
	if ( tree )
		free(tree);

	if ( dtree )
		cm->freeLine(dtree);
}


//void printTree15(int *iptr, float *fptr)
//{
//	printf("%d %d %d    %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f\n",
//		*iptr, *(iptr+1), *(iptr+2),
//		*fptr, *(fptr+2), *(fptr+3), *(fptr+4), *(fptr+5), *(fptr+6), *(fptr+7), *(fptr+8));
//}

// Default constructor.
svlCudaDTree15::svlCudaDTree15()
{
	tree = NULL;
	treei = NULL;
	treef = NULL;
	dtree = NULL;
	dtreei = NULL;
	dtreef = NULL;
}   

// Read in model from file.
svlCudaDTree15::svlCudaDTree15(const char *file)
{
	tree = NULL;
	treei = NULL;
	treef = NULL;
	dtree = NULL;
	dtreei = NULL;
	dtreef = NULL;

	FILE *fp = fopen(file,"rb");
	if ( !fp ) {
		printf("Initialized svlCudaDTree15 with NULL FILE: %s.\n", file);
		return;
	}

	// Read in total tree.
	fp = fopen(file,"rb");
	fread(&n_trees,sizeof(int),1,fp);
	treei = (int*)malloc(sizeof(int)*n_trees*7);
	treef = (float*)malloc(sizeof(float)*n_trees*21);
	fread(treei,sizeof(int),n_trees*7,fp);
	fread(treef,sizeof(float),n_trees*21,fp);
	fclose(fp);

	//// Debug print
	//for ( int i=0; i<n_trees; ++i )
	//	printTree15(itree + i*3, ftree + i*8);

	// Allocate device memory and transfer in.
	dtreei = cm->allocLine(n_trees*7, "decision tree ints");
	dtreef = cm->allocLine(n_trees*21, "decision tree floats");
	cm->lineToGPU(dtreei, (float*)treei);
	cm->lineToGPU(dtreef, treef);
}

svlCudaDTree15::~svlCudaDTree15()
{
	// Free host memory.
	if ( tree ) {
		free(tree);
		tree = NULL;
	}
	if ( treei ) {
		free(treei);
		treei = NULL;
		free(treef);
		treef = NULL;
	}
	if ( dtreei ) {
		cm->freeLine(dtreei);
		dtreei = NULL;
		cm->freeLine(dtreef);
		dtreef = NULL;
	}
}


void svlCudaDTree15::hostEvaluate(const float *features, float *invalid, float *res, int feature_len, int feature_stride, int n_fvs) const
{
	// Clear invalid bits.
	if ( invalid )
		memset(invalid, 0, sizeof(float)*n_fvs);

	// Compute response of tree on host.
	for ( int i=0; i<n_fvs; ++i ) {
		float gf = 0.0f; // Response over trees.
		const float *data = features + feature_stride * i;

		// TODO paulb: Restore this.
		/*for ( int j=0; j<feature_len; ++j )
			if ( isinf(data[j]) || isnan(data[j]) )
				invalid[i] = 1.0f;*/

		for ( int t=0; t<n_trees; ++t ) {
			int tixi = 7*t, tixf = 21*t;

			if ( data[ treei[tixi] ] <= treef[tixf] ) {
				// L
				if ( treei[tixi + 1] < 0 )
					gf += treef[tixf + 2];
				else if ( data[ treei[tixi + 1] ] <= treef[tixf + 1] ) {
					// LL
					if ( treei[tixi + 2] < 0 )
						gf += treef[tixf + 4];
					else if ( data[ treei[tixi + 2] ] <= treef[tixf + 3] )
						gf += treef[tixf + 5];
					else
						gf += treef[tixf + 6];
				} else {
					// LR
					if ( treei[tixi + 3] < 0 )
						gf += treef[tixf + 8];
					else if ( data[ treei[tixi + 3] ] <= treef[tixf + 7] )
						gf += treef[tixf + 9];
					else
						gf += treef[tixf + 10];
				}
			} else {
				// R
				if ( treei[tixi + 4] < 0 )
					gf += treef[tixf + 12];
				else if ( data[ treei[tixi + 4] ] <= treef[tixf + 11] ) {
					// RL
					if ( treei[tixi + 5] < 0 )
						gf += treef[tixf + 14];
					else if ( data[ treei[tixi + 5] ] <= treef[tixf + 13] )
						gf += treef[tixf + 15];
					else
						gf += treef[tixf + 16];
				} else {
					// RR
					if ( treei[tixi + 6] < 0 )
						gf += treef[tixf + 18];
					else if ( data[ treei[tixi + 6] ] <= treef[tixf + 17] )
						gf += treef[tixf + 19];
					else
						gf += treef[tixf + 20];
				}
			}
		}
		
		// Write out result.
		res[i] = gf;
	}
}


